import importlib

import pytest
from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import User
from django.contrib.messages import get_messages
from django.urls import reverse

from django_microsoft_sso import conf
from django_microsoft_sso.main import MicrosoftAuth
from django_microsoft_sso.tests.conftest import SECRET_PATH

ROUTE_NAME = "django_microsoft_sso:oauth_callback"


pytestmark = pytest.mark.django_db()


class MyBackend(ModelBackend):
    """Simple test for custom authentication backend"""

    def authenticate(self, request, username=None, password=None, **kwargs):
        return super().authenticate(request, username, password, **kwargs)


def test_start_login(client, mocker):
    # Arrange
    flow_mock = mocker.patch.object(MicrosoftAuth, "auth")
    flow_mock.initiate_auth_code_flow.return_value = {
        "state": "foo",
        "auth_uri": "https://foo/bar",
    }

    # Act
    url = reverse("django_microsoft_sso:oauth_start_login") + "?next=/secret/"
    response = client.get(url)

    # Assert
    assert response.status_code == 302
    assert client.session["sso_next_url"] == SECRET_PATH


def test_start_login_none_next_param(client, mocker):
    # Arrange
    flow_mock = mocker.patch.object(MicrosoftAuth, "auth")
    flow_mock.initiate_auth_code_flow.return_value = {
        "state": "foo",
        "auth_uri": "https://foo/bar",
    }

    # Act
    url = reverse("django_microsoft_sso:oauth_start_login")
    response = client.get(url)

    # Assert
    assert response.status_code == 302
    assert client.session["sso_next_url"] == reverse(conf.MICROSOFT_SSO_NEXT_URL)


@pytest.mark.parametrize(
    "test_parameter",
    [
        "bad-domain.com/secret/",
        "www.bad-domain.com/secret/",
        "//bad-domain.com/secret/",
        "http://bad-domain.com/secret/",
        "https://malicious.example.com/secret/",
    ],
)
def test_exploit_redirect(client, mocker, test_parameter):
    # Arrange
    flow_mock = mocker.patch.object(MicrosoftAuth, "auth")
    flow_mock.initiate_auth_code_flow.return_value = {
        "state": "foo",
        "auth_uri": "https://foo/bar",
    }

    # Act
    url = reverse("django_microsoft_sso:oauth_start_login") + f"?next={test_parameter}"
    response = client.get(url)

    # Assert
    assert response.status_code == 302
    assert client.session["sso_next_url"] == SECRET_PATH


def test_microsoft_sso_disabled(settings, client):
    # Arrange
    from django_microsoft_sso import conf

    settings.MICROSOFT_SSO_ENABLED = False
    importlib.reload(conf)

    # Act
    response = client.get(reverse(ROUTE_NAME))

    # Assert
    assert response.status_code == 302
    assert User.objects.count() == 0
    assert "Microsoft SSO not enabled." in [
        m.message for m in get_messages(response.wsgi_request)
    ]


def test_missing_code(client):
    # Arrange
    importlib.reload(conf)

    # Act
    response = client.get(reverse(ROUTE_NAME))

    # Assert
    assert response.status_code == 302
    assert User.objects.count() == 0
    assert "Authorization Code not received from SSO." in [
        m.message for m in get_messages(response.wsgi_request)
    ]


@pytest.mark.parametrize("querystring", ["?code=1234", "?code=1234&state=bad_dog"])
def test_bad_state(client, querystring):
    # Arrange
    importlib.reload(conf)
    session = client.session
    session.update({"sso_state": "good_dog"})
    session.save()

    # Act
    url = reverse(ROUTE_NAME) + querystring
    response = client.get(url)

    # Assert
    assert response.status_code == 302
    assert User.objects.count() == 0
    assert "State Mismatch. Time expired?" in [
        m.message for m in get_messages(response.wsgi_request)
    ]


def test_invalid_email(client_with_session, settings, callback_url, microsoft_response):
    # Arrange
    from django_microsoft_sso import conf

    settings.MICROSOFT_SSO_ALLOWABLE_DOMAINS = ["foobar.com"]
    importlib.reload(conf)

    # Act
    response = client_with_session.get(callback_url)

    # Assert
    assert response.status_code == 302
    assert User.objects.count() == 0
    assert (
        f"Email address not allowed: {microsoft_response['mail']}. "
        f"Please contact your administrator."
        in [m.message for m in get_messages(response.wsgi_request)]
    )


def test_inactive_user(client_with_session, callback_url, microsoft_response):
    # Arrange
    User.objects.create(
        username=microsoft_response["mail"],
        email=microsoft_response["mail"],
        is_active=False,
    )

    # Act
    response = client_with_session.get(callback_url)

    # Assert
    assert response.status_code == 302
    assert User.objects.count() == 1
    assert User.objects.get(email=microsoft_response["mail"]).is_active is False


def test_new_user_login(client_with_session, callback_url, settings, mocker):
    # Arrange
    flow_mock = mocker.patch.object(MicrosoftAuth, "auth")
    flow_mock.acquire_token_by_auth_code_flow.return_value = {"access_token": "foo"}
    User.objects.all().delete()
    assert User.objects.count() == 0
    settings.MICROSOFT_SSO_ALLOWABLE_DOMAINS = ["dailyplanet.com"]
    importlib.reload(conf)

    # Act
    response = client_with_session.get(callback_url)

    # Assert
    assert response.status_code == 302
    assert User.objects.count() == 1
    assert response.url == SECRET_PATH
    assert response.wsgi_request.user.is_authenticated is True


def test_existing_user_login(
    client_with_session, settings, microsoft_response, callback_url, mocker
):
    # Arrange
    from django_microsoft_sso import conf

    flow_mock = mocker.patch.object(MicrosoftAuth, "auth")
    flow_mock.acquire_token_by_auth_code_flow.return_value = {"access_token": "foo"}

    existing_user = User.objects.create(
        username=microsoft_response["mail"],
        email=microsoft_response["mail"],
        is_active=True,
    )

    settings.MICROSOFT_SSO_ALLOWABLE_DOMAINS = ["dailyplanet.com"]
    settings.MICROSOFT_SSO_AUTO_CREATE_USERS = False
    importlib.reload(conf)

    # Act
    response = client_with_session.get(callback_url)

    # Assert
    assert response.status_code == 302
    assert User.objects.count() == 1
    assert response.url == SECRET_PATH
    assert response.wsgi_request.user.is_authenticated is True
    assert response.wsgi_request.user.email == existing_user.email


def test_missing_user_login(client_with_session, settings, callback_url):
    # Arrange
    from django_microsoft_sso import conf

    settings.MICROSOFT_SSO_AUTO_CREATE_USERS = False
    importlib.reload(conf)

    # Act
    response = client_with_session.get(callback_url)

    # Assert
    assert response.status_code == 302
    assert User.objects.count() == 0
    assert response.url == "/admin/"
    assert response.wsgi_request.user.is_authenticated is False



import pytest
from django.contrib.sites.models import Site
from msal.authority import AZURE_PUBLIC, AuthorityBuilder

from django_microsoft_sso import conf
from django_microsoft_sso.main import MicrosoftAuth

pytestmark = pytest.mark.django_db


def test_scopes(callback_request):
    # Arrange
    ms = MicrosoftAuth(callback_request)

    # Assert
    assert ms.scopes == conf.MICROSOFT_SSO_SCOPES


def test_get_redirect_uri_with_http(callback_request, monkeypatch):
    # Arrange
    expected_scheme = "http"
    monkeypatch.setattr(conf, "MICROSOFT_SSO_CALLBACK_DOMAIN", None)
    current_site_domain = Site.objects.get_current().domain

    # Act
    ms = MicrosoftAuth(callback_request)

    # Assert
    assert (
        ms.get_redirect_uri()
        == f"{expected_scheme}://{current_site_domain}/microsoft_sso/callback/"
    )


def test_get_redirect_uri_with_reverse_proxy(
    callback_request_from_reverse_proxy, monkeypatch
):
    # Arrange
    expected_scheme = "https"
    monkeypatch.setattr(conf, "MICROSOFT_SSO_CALLBACK_DOMAIN", None)
    current_site_domain = Site.objects.get_current().domain

    # Act
    ms = MicrosoftAuth(callback_request_from_reverse_proxy)

    # Assert
    assert (
        ms.get_redirect_uri()
        == f"{expected_scheme}://{current_site_domain}/microsoft_sso/callback/"
    )


def test_redirect_uri_with_custom_domain(callback_request_from_reverse_proxy, monkeypatch):
    # Arrange
    monkeypatch.setattr(conf, "MICROSOFT_SSO_CALLBACK_DOMAIN", "my-other-domain.com")

    # Act
    ms = MicrosoftAuth(callback_request_from_reverse_proxy)

    # Assert
    assert ms.get_redirect_uri() == "https://my-other-domain.com/microsoft_sso/callback/"


@pytest.mark.parametrize(
    "data, expect_raise",
    [
        ("https://login.microsoftonline.com/contoso", False),
        (AuthorityBuilder(AZURE_PUBLIC, "contoso.onmicrosoft.com"), False),
        (None, False),
        ("contoso", True),
    ],
)
def test_custom_authorities(
    data, expect_raise, callback_request_from_reverse_proxy, monkeypatch
):
    # Arrange
    monkeypatch.setattr(conf, "MICROSOFT_SSO_AUTHORITY", data)

    # Act
    ms = MicrosoftAuth(callback_request_from_reverse_proxy)

    # Assert
    if expect_raise:
        with pytest.raises(ValueError):
            ms.get_authority()
    else:
        assert ms.get_authority() == data



import importlib


def test_conf_from_settings(settings):
    # Arrange
    settings.MICROSOFT_SSO_ENABLED = False

    # Act
    from django_microsoft_sso import conf

    importlib.reload(conf)

    # Assert
    assert conf.MICROSOFT_SSO_ENABLED is False



from django.http import HttpRequest

from django_microsoft_sso.models import User


def pre_login_user(user: User, request: HttpRequest) -> None:
    """
    Callback function called after user is created/retrieved but before logged in.
    """


def pre_create_user(ms_user_info: dict, request: HttpRequest) -> dict | None:
    """
    Callback function called before user is created.

    params:
        ms_user_info: dict containing user info received from Microsoft Graph.
        request: HttpRequest object.

    return: dict content to be passed to User.objects.create() as `defaults` argument.
    """
    return {}


def pre_validate_user(ms_user_info: dict, request: HttpRequest) -> bool:
    """
    Callback function called before user is validated.

    Must return a boolean to indicate if user is valid to login.

    params:
        ms_user_info: dict containing user info received from Microsoft Graph.
        request: HttpRequest object.
    """
    return True



import pytest

from django_microsoft_sso.main import UserHelper

pytestmark = pytest.mark.django_db


def test_microsoft_sso_model(microsoft_response, callback_request, settings):
    # Act
    helper = UserHelper(microsoft_response, callback_request)
    user = helper.get_or_create_user()

    # Assert
    assert user.microsoftssouser.microsoft_id == microsoft_response["id"]
    assert user.microsoftssouser.picture_raw == microsoft_response["picture_raw_data"]
    assert user.microsoftssouser.locale == microsoft_response["preferredLanguage"]



import importlib
from urllib.parse import urlparse

from django.contrib.auth import login
from django.contrib.auth.views import LogoutView
from django.http import HttpRequest, HttpResponseRedirect
from django.shortcuts import resolve_url
from django.template.response import TemplateResponse
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.decorators.http import require_http_methods
from loguru import logger

from django_microsoft_sso import conf
from django_microsoft_sso.main import MicrosoftAuth, UserHelper
from django_microsoft_sso.utils import send_message, show_credential


@require_http_methods(["GET"])
def start_login(request: HttpRequest) -> HttpResponseRedirect:
    # Get the next url
    next_param = request.GET.get(key="next")
    if next_param:
        clean_param = (
            next_param
            if next_param.startswith("http") or next_param.startswith("/")
            else f"//{next_param}"
        )
    else:
        clean_param = reverse(conf.MICROSOFT_SSO_NEXT_URL)
    next_path = urlparse(clean_param).path

    ms_auth = MicrosoftAuth(request)
    ms_auth.initiate()

    # Save data on Session
    if not request.session.session_key:
        request.session.create()
    request.session.set_expiry(conf.MICROSOFT_SSO_TIMEOUT * 60)
    request.session["msal_graph_info"] = ms_auth.result
    request.session["sso_next_url"] = next_path
    request.session.save()

    # Redirect User
    return HttpResponseRedirect(ms_auth.get_auth_uri())


@require_http_methods(["GET"])
def callback(request: HttpRequest) -> HttpResponseRedirect:
    login_failed_url = reverse(conf.MICROSOFT_SSO_LOGIN_FAILED_URL)
    microsoft = MicrosoftAuth(request)
    code = request.GET.get("code")
    state = request.GET.get("state")

    # Check if Microsoft SSO is enabled
    if not conf.MICROSOFT_SSO_ENABLED:
        send_message(request, _("Microsoft SSO not enabled."))
        return HttpResponseRedirect(login_failed_url)

    # First, check for authorization code
    if not code:
        send_message(request, _("Authorization Code not received from SSO."))
        return HttpResponseRedirect(login_failed_url)

    # Then, check the state.
    request_state = request.session.get("msal_graph_info", {}).get("state")
    next_url = request.session.get("sso_next_url")

    if not request_state or state != request_state:
        send_message(request, _("State Mismatch. Time expired?"))
        return HttpResponseRedirect(login_failed_url)

    # Get Access Token from Microsoft Graph
    auth_result = microsoft.get_user_token()
    if not auth_result:
        send_message(request, _("Access Token not received from SSO."))
        return HttpResponseRedirect(login_failed_url)
    if "error" in auth_result:
        send_message(
            request, _(f"Authorization Error received from SSO: {auth_result['error']}.")
        )
        if auth_result["error"] == "invalid_client":
            send_message(
                request, _("Please check your Client Credentials for MS Entra App.")
            )
            logger.debug(
                f"MICROSOFT_SSO_APPLICATION_ID: "
                f"{show_credential(conf.MICROSOFT_SSO_APPLICATION_ID)}"
            )
            logger.debug(
                f"MICROSOFT_SSO_CLIENT_SECRET: "
                f"{show_credential(conf.MICROSOFT_SSO_CLIENT_SECRET)}"
            )
        return HttpResponseRedirect(login_failed_url)

    try:
        user_result = microsoft.get_user_info()
    except Exception as error:
        send_message(request, _(f"Error while processing callback from SSO: {error}."))
        return HttpResponseRedirect(login_failed_url)

    user_helper = UserHelper(user_result, request)

    # Run Pre-Validate Callback
    module_path = ".".join(conf.MICROSOFT_SSO_PRE_VALIDATE_CALLBACK.split(".")[:-1])
    pre_validate_fn = conf.MICROSOFT_SSO_PRE_VALIDATE_CALLBACK.split(".")[-1]
    module = importlib.import_module(module_path)
    user_is_valid = getattr(module, pre_validate_fn)(user_result, request)

    # Check if User Info is valid to login
    if not user_helper.email_is_valid or not user_is_valid:
        send_message(
            request,
            _(
                f"Email address not allowed: {user_helper.user_email}. "
                f"Please contact your administrator."
            ),
        )
        return HttpResponseRedirect(login_failed_url)

    # Add Access Token in Session
    if conf.MICROSOFT_SSO_SAVE_ACCESS_TOKEN:
        request.session["microsoft_sso_access_token"] = microsoft.token_info["access_token"]

    # Run Pre-Create Callback
    module_path = ".".join(conf.MICROSOFT_SSO_PRE_CREATE_CALLBACK.split(".")[:-1])
    pre_login_fn = conf.MICROSOFT_SSO_PRE_CREATE_CALLBACK.split(".")[-1]
    module = importlib.import_module(module_path)
    extra_users_args = getattr(module, pre_login_fn)(user_result, request)

    # Get or Create User
    if conf.MICROSOFT_SSO_AUTO_CREATE_USERS:
        user = user_helper.get_or_create_user(extra_users_args)
    else:
        user = user_helper.find_user()

    if not user or not user.is_active:
        failed_login_message = (
            f"User not found - UPN: '{user_result['userPrincipalName']}', "
            f"Email: '{user_result['mail']}'"
        )
        if not user and not conf.MICROSOFT_SSO_AUTO_CREATE_USERS:
            failed_login_message += ". Auto-Create is disabled."

        if user and not user.is_active:
            failed_login_message = (
                f"User is not active: '{user_result['userPrincipalName']}'"
            )

        if conf.MICROSOFT_SSO_SHOW_FAILED_LOGIN_MESSAGE:
            send_message(request, _(failed_login_message), level="warning")
        else:
            logger.warning(failed_login_message)

        return HttpResponseRedirect(login_failed_url)

    # Save Session
    request.session.save()

    # Run Pre-Login Callback
    module_path = ".".join(conf.MICROSOFT_SSO_PRE_LOGIN_CALLBACK.split(".")[:-1])
    pre_login_fn = conf.MICROSOFT_SSO_PRE_LOGIN_CALLBACK.split(".")[-1]
    module = importlib.import_module(module_path)
    getattr(module, pre_login_fn)(user, request)

    # Login User
    login(request, user, conf.MICROSOFT_SSO_AUTHENTICATION_BACKEND)
    request.session.set_expiry(conf.MICROSOFT_SSO_SESSION_COOKIE_AGE)

    return HttpResponseRedirect(next_url or reverse(conf.MICROSOFT_SSO_NEXT_URL))


@require_http_methods(["POST", "OPTIONS"])
def microsoft_slo_view(request: HttpRequest) -> TemplateResponse:
    """
    Logout the User from Microsoft SSO and Django.

    Use this View for your logout URL.

    """

    if conf.MICROSOFT_SLO_ENABLED and conf.MICROSOFT_SSO_ENABLED:
        microsoft = MicrosoftAuth(request)
        homepage = resolve_url(
            conf.MICROSOFT_SSO_LOGOUT_REDIRECT_PATH
        )  # Default: "admin:index"
        if not homepage.startswith("http"):
            homepage = request.build_absolute_uri(homepage)
        next_page = microsoft.get_logout_url(homepage=homepage)
        return LogoutView.as_view(next_page=next_page)(request)
    return LogoutView.as_view()(request)



import uuid
from dataclasses import dataclass
from typing import Any
from urllib.parse import urlparse

import httpx
import msal
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AbstractUser
from django.contrib.sites.shortcuts import get_current_site
from django.db.models import Field, Model, Q
from django.http import HttpRequest
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from loguru import logger
from msal import ConfidentialClientApplication
from msal.authority import AuthorityBuilder

from django_microsoft_sso import conf
from django_microsoft_sso.models import MicrosoftSSOUser

STATE = str(uuid.uuid4())


@dataclass
class MicrosoftAuth:
    request: HttpRequest
    _auth: ConfidentialClientApplication = None
    result: dict[Any, Any] | None = None
    token_info: dict[Any, Any] | None = None

    @property
    def scopes(self) -> list[str]:
        return conf.MICROSOFT_SSO_SCOPES

    def get_netloc(self):
        if conf.MICROSOFT_SSO_CALLBACK_DOMAIN:
            logger.debug("Find Netloc using MICROSOFT_SSO_CALLBACK_DOMAIN")
            return conf.MICROSOFT_SSO_CALLBACK_DOMAIN

        site = get_current_site(self.request)
        logger.debug("Find Netloc using Site domain")
        return site.domain

    def get_redirect_uri(self) -> str:
        if "HTTP_X_FORWARDED_PROTO" in self.request.META:
            scheme = self.request.META["HTTP_X_FORWARDED_PROTO"]
        else:
            scheme = self.request.scheme
        netloc = self.get_netloc()
        path = reverse("django_microsoft_sso:oauth_callback")
        callback_uri = f"{scheme}://{netloc}{path}"
        logger.debug(f"Callback URI: {callback_uri}")
        return callback_uri

    @property
    def auth(self) -> ConfidentialClientApplication:
        if not self._auth:
            authority = self.get_authority()
            self._auth = msal.ConfidentialClientApplication(
                client_id=conf.MICROSOFT_SSO_APPLICATION_ID,
                client_credential=conf.MICROSOFT_SSO_CLIENT_SECRET,
                authority=authority,
            )
        return self._auth

    @staticmethod
    def get_authority():
        if conf.MICROSOFT_SSO_AUTHORITY is None or isinstance(
            conf.MICROSOFT_SSO_AUTHORITY, AuthorityBuilder
        ):
            return conf.MICROSOFT_SSO_AUTHORITY

        authority_value = conf.MICROSOFT_SSO_AUTHORITY
        if not isinstance(authority_value, str):
            raise ValueError(
                "MICROSOFT_SSO_AUTHORITY must be a valid URL or an AuthorityBuilder object"
            )

        if isinstance(authority_value, str):
            data = urlparse(authority_value)
            if not data.scheme or not data.netloc:
                raise ValueError(
                    "MICROSOFT_SSO_AUTHORITY must be a valid URL "
                    "or an AuthorityBuilder object"
                )
        return authority_value

    def get_user_info(self):
        graph_url = "https://graph.microsoft.com/v1.0/me"
        token = self.token_info["access_token"]
        headers = {"Authorization": f"Bearer {token}"}
        response = httpx.get(
            graph_url, headers=headers, timeout=conf.MICROSOFT_SSO_GRAPH_TIMEOUT
        )
        user_info = response.json()
        response.raise_for_status()

        # Get Email Verified Flag
        graph_url = "https://graph.microsoft.com/v1.0/users/{}?$select=mailVerified".format(
            user_info["id"]
        )
        response = httpx.get(
            graph_url, headers=headers, timeout=conf.MICROSOFT_SSO_GRAPH_TIMEOUT
        )
        if response.status_code == 200:
            user_info.update({"email_verified": response.json().get("mailVerified", False)})

        # Get Picture Data
        graph_url = "https://graph.microsoft.com/v1.0/me/photo/$value"
        response = httpx.get(
            graph_url, headers=headers, timeout=conf.MICROSOFT_SSO_GRAPH_TIMEOUT
        )
        if response.status_code == 200:
            user_info.update({"picture_raw_data": response.content})

        return user_info

    def get_auth_uri(self):
        return self.result["auth_uri"]

    def get_user_token(self):
        self.token_info = self.auth.acquire_token_by_auth_code_flow(
            auth_code_flow=self.request.session["msal_graph_info"],
            auth_response={
                "code": self.request.GET.get("code"),
                "state": self.request.GET.get("state"),
                "session_state": self.request.GET.get("session_state"),
            },
        )
        return self.token_info

    def initiate(
        self, custom_scopes: list[str] | None = None, redirect_uri: str | None = None
    ) -> dict:
        self.result = self.auth.initiate_auth_code_flow(
            scopes=custom_scopes or settings.MICROSOFT_SSO_SCOPES,
            redirect_uri=redirect_uri or self.get_redirect_uri(),
            state=STATE,
        )
        return self.result

    @staticmethod
    def get_logout_url(homepage):
        return (
            f"https://login.microsoftonline.com/common/oauth2/v2.0/"
            f"logout?post_logout_redirect_uri={homepage}"
        )


@dataclass
class UserHelper:
    user_info: dict[Any, Any]
    request: Any
    user_changed: bool = False

    @property
    def user_email(self) -> str:
        user_email = self.user_info.get("mail") or ""
        return user_email.lower()  # Ensure email is lowercase

    @property
    def user_principal_name(self) -> str:
        return self.user_info["userPrincipalName"].lower()

    @property
    def user_model(self) -> AbstractUser | Model:
        return get_user_model()

    @property
    def username_field(self) -> Field:
        return self.user_model._meta.get_field(self.user_model.USERNAME_FIELD)

    @property
    def email_is_valid(self) -> bool:
        user_email_domain = self.user_email.split("@")[-1]
        valid_domain = False
        for email_domain in conf.MICROSOFT_SSO_ALLOWABLE_DOMAINS:
            if user_email_domain in email_domain:
                valid_domain = True
        email_verified = self.user_info.get("verified_email", None)
        if email_verified is not None and not email_verified:
            logger.debug(f"Email {self.user_email} is not verified.")
        return valid_domain

    def get_or_create_user(self, extra_users_args: dict | None = None):
        user_defaults = extra_users_args or {}

        if conf.MICROSOFT_SSO_UNIQUE_EMAIL:
            if not self.user_email:
                raise ValueError("User email not found in Tenant data.")
            if self.username_field.name not in user_defaults:
                user_defaults[self.username_field.name] = self.user_principal_name

            user, created = self.user_model.objects.get_or_create(
                email__iexact=self.user_email, defaults=user_defaults
            )
        else:
            user_defaults["email"] = self.user_email

            # Find searching User Principal Name in MicrosoftSSOUser
            # For existing databases prior to this version, this field can be empty
            query = self.user_model.objects.filter(
                microsoftssouser__user_principal_name__iexact=self.user_principal_name
            )
            if query.exists():
                user = query.get()
                created = False
            else:
                username = user_defaults.pop(
                    self.username_field.name, self.user_principal_name
                )
                if "email" not in user_defaults:
                    user_defaults["email"] = self.user_email
                if self.username_field.attname not in user_defaults:
                    user_defaults[self.username_field.attname] = username
                query = {
                    f"{self.username_field.attname}__iexact": username,
                    "defaults": user_defaults,
                }
                user, created = self.user_model.objects.get_or_create(**query)

        self.check_first_super_user(user)
        self.check_for_update(created, user)
        if self.user_changed:
            user.save()

        if conf.MICROSOFT_SSO_SAVE_BASIC_MICROSOFT_INFO:
            MicrosoftSSOUser.objects.update_or_create(
                user=user,
                defaults={
                    "microsoft_id": self.user_info["id"],
                    "picture_raw": self.user_info.get("picture_raw_data"),
                    "locale": self.user_info.get("preferredLanguage"),
                    "user_principal_name": self.user_principal_name,
                },
            )

        return user

    def check_for_update(self, created, user):
        if created or conf.MICROSOFT_SSO_ALWAYS_UPDATE_USER_DATA:
            self.check_for_permissions(user)
            user.first_name = self.user_info.get("givenName") or ""
            user.last_name = self.user_info.get("surname") or ""
            user.email = self.user_email
            user.set_unusable_password()
            self.user_changed = True

    def check_first_super_user(self, user):
        if conf.MICROSOFT_SSO_AUTO_CREATE_FIRST_SUPERUSER:
            superuser_exists = self.user_model.objects.filter(is_superuser=True).exists()
            if not superuser_exists:
                message_text = _(
                    f"MICROSOFT_SSO_AUTO_CREATE_FIRST_SUPERUSER is True. "
                    f"Adding SuperUser status to email: {self.user_principal_name}"
                )
                messages.add_message(self.request, messages.INFO, message_text)
                logger.warning(message_text)
                user.is_superuser = True
                user.is_staff = True
                self.user_changed = True

    def check_for_permissions(self, user):
        username = getattr(user, self.username_field.name, None)
        if (
            user.email in conf.MICROSOFT_SSO_STAFF_LIST
            or username in conf.MICROSOFT_SSO_STAFF_LIST
            or "*" in conf.MICROSOFT_SSO_STAFF_LIST
        ):
            message_text = _(
                f"User: {self.user_principal_name} in MICROSOFT_SSO_STAFF_LIST. "
                f"Added Staff Permission."
            )
            messages.add_message(self.request, messages.INFO, message_text)
            logger.debug(message_text)
            user.is_staff = True
        if (
            user.email in conf.MICROSOFT_SSO_SUPERUSER_LIST
            or username in conf.MICROSOFT_SSO_SUPERUSER_LIST
        ):
            message_text = _(
                f"User: {self.user_principal_name} in MICROSOFT_SSO_SUPERUSER_LIST. "
                f"Added SuperUser Permission."
            )
            messages.add_message(self.request, messages.INFO, message_text)
            logger.debug(message_text)
            user.is_superuser = True
            user.is_staff = True

    def find_user(self):
        if conf.MICROSOFT_SSO_UNIQUE_EMAIL:
            query = self.user_model.objects.filter(email__iexact=self.user_email)
        else:
            username_query = {
                f"{self.username_field.attname}__iexact": self.user_principal_name
            }
            query = self.user_model.objects.filter(
                Q(microsoftssouser__user_principal_name__iexact=self.user_principal_name)
                | Q(**username_query)
            )
        if query.exists():
            return query.get()



from django import template

register = template.Library()


@register.simple_tag
def define_show_form() -> bool:
    from django.conf import settings

    return getattr(settings, "SSO_SHOW_FORM_ON_ADMIN_PAGE", True)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DjangoMicrosoftSsoConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_microsoft_sso"
    verbose_name = _("Microsoft SSO User")

    def ready(self):
        import django_microsoft_sso.templatetags  # noqa



from typing import Optional, Type

from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.models import AbstractUser

from django_microsoft_sso import conf
from django_microsoft_sso.models import MicrosoftSSOUser

if conf.MICROSOFT_SSO_ENABLED:
    admin.site.login_template = "microsoft_sso/login.html"


def get_current_user_and_admin() -> (
    tuple[AbstractUser, Optional[UserAdmin], Type[UserAdmin]]
):
    """Get the current user model and last admin class.

    For user model, we use the get_user_model() function.
    For the last admin class registered, we use the
    admin.site._registry.get(user_model) function or default `UserAdmin`

    """

    user_model = get_user_model()
    existing_user_admin = admin.site._registry.get(user_model)
    user_admin_model = (
        UserAdmin if existing_user_admin is None else existing_user_admin.__class__
    )
    return user_model, existing_user_admin, user_admin_model


class MicrosoftSSOInlineAdmin(admin.StackedInline):
    model = MicrosoftSSOUser
    readonly_fields = ("microsoft_id", "picture", "user_principal_name")
    extra = 0

    def has_add_permission(self, request, obj):
        return False


CurrentUserModel, last_admin, LastUserAdmin = get_current_user_and_admin()


if admin.site.is_registered(CurrentUserModel):
    admin.site.unregister(CurrentUserModel)


@admin.register(MicrosoftSSOUser)
class MicrosoftSSOAdmin(admin.ModelAdmin):
    list_display = ("user", "microsoft_id")
    readonly_fields = ("microsoft_id", "picture", "user_principal_name")

    def has_add_permission(self, request):
        return False


@admin.register(CurrentUserModel)
class MicrosoftSSOUserAdmin(LastUserAdmin):
    model = CurrentUserModel
    inlines = (
        tuple(set(list(last_admin.inlines) + [MicrosoftSSOInlineAdmin]))
        if last_admin
        else (MicrosoftSSOInlineAdmin,)
    )



import importlib
from copy import deepcopy
from urllib.parse import quote, urlencode

import pytest
from django.contrib.messages.storage.fallback import FallbackStorage
from django.contrib.sessions.middleware import SessionMiddleware
from django.urls import reverse

from django_microsoft_sso import conf
from django_microsoft_sso.main import MicrosoftAuth

SECRET_PATH = "/secret/"


@pytest.fixture
def query_string():
    return urlencode(
        {
            "code": "12345",
            "state": "foo",
            "scope": " ".join(conf.MICROSOFT_SSO_SCOPES),
            "hd": "example.com",
            "prompt": "consent",
        },
        quote_via=quote,
    )


@pytest.fixture
def microsoft_response():
    return {
        "@odata.context": "https://graph.microsoft.com/v1.0/$metadata#users/$entity",
        "businessPhones": [],
        "displayName": "Clark Kent",
        "givenName": "Clark",
        "jobTitle": None,
        "mail": "kalel@dailyplanet.com",
        "mobilePhone": None,
        "officeLocation": None,
        "preferredLanguage": "en-US",
        "surname": "Kent",
        "userPrincipalName": "kalel@dailyplanet.com",
        "id": "291azxdc-8e44-aa13-119b-60adddsss5e99",
        "picture_raw_data": b"foo",
        "verified_email": True,
    }


@pytest.fixture
def microsoft_response_update():
    return {
        "@odata.context": "https://graph.microsoft.com/v1.0/$metadata#users/$entity",
        "businessPhones": [],
        "displayName": "Bruce Wayne",
        "givenName": "Bruce",
        "jobTitle": None,
        "mail": "kalel@dailyplanet.com",
        "mobilePhone": None,
        "officeLocation": None,
        "preferredLanguage": "en-US",
        "surname": "Wayne",
        "userPrincipalName": "kalel@dailyplanet.com",
        "id": "291azxdc-8e44-aa13-119b-60adddsss5e99",
        "picture_raw_data": b"foo",
        "verified_email": True,
    }


@pytest.fixture
def callback_request(rf, query_string):
    request = rf.get(f"/microsoft_sso/callback/?{query_string}")
    middleware = SessionMiddleware(get_response=lambda req: None)
    middleware.process_request(request)
    request.session.save()
    messages = FallbackStorage(request)
    setattr(request, "_messages", messages)
    return request


@pytest.fixture
def callback_request_from_reverse_proxy(rf, query_string):
    request = rf.get(
        f"/microsoft_sso/callback/?{query_string}", HTTP_X_FORWARDED_PROTO="https"
    )
    middleware = SessionMiddleware(get_response=lambda req: None)
    middleware.process_request(request)
    request.session.save()
    messages = FallbackStorage(request)
    setattr(request, "_messages", messages)
    return request


@pytest.fixture
def callback_request_with_state(callback_request):
    request = deepcopy(callback_request)
    request.session["msal_graph_info"] = {"state": "foo"}
    request.session["sso_next_url"] = "/secret/"
    return request


@pytest.fixture
def client_with_session(client, settings, mocker, microsoft_response):
    settings.MICROSOFT_SSO_ALLOWABLE_DOMAINS = ["example.com"]
    settings.MICROSOFT_SSO_PRE_LOGIN_CALLBACK = "django_microsoft_sso.hooks.pre_login_user"
    settings.MICROSOFT_SSO_PRE_CREATE_CALLBACK = (
        "django_microsoft_sso.hooks.pre_create_user"
    )
    settings.MICROSOFT_SSO_PRE_VALIDATE_CALLBACK = (
        "django_microsoft_sso.hooks.pre_validate_user"
    )
    importlib.reload(conf)
    session = client.session
    session.update({"msal_graph_info": {"state": "foo"}, "sso_next_url": SECRET_PATH})
    session.save()
    m = mocker.patch("django_microsoft_sso.main.ConfidentialClientApplication")
    m.acquire_token_by_auth_code_flow.return_value = {"access_token": "foo"}
    m.initiate_auth_code_flow.return_value = {"state": "foo"}
    mocker.patch.object(MicrosoftAuth, "auth", return_value=m)
    mocker.patch.object(MicrosoftAuth, "get_user_info", return_value=microsoft_response)
    yield client


@pytest.fixture
def callback_url(query_string):
    return f"{reverse('django_microsoft_sso:oauth_callback')}?{query_string}"



from django.contrib import messages
from loguru import logger

from django_microsoft_sso import conf


def send_message(request, message, level: str = "error"):
    getattr(logger, level.lower())(message)
    if conf.MICROSOFT_SSO_ENABLE_MESSAGES:
        messages.add_message(request, getattr(messages, level.upper()), message)


def show_credential(credential):
    return f"{credential[:5]}...{credential[-5:]}"



from base64 import b64encode

from django.contrib.auth import get_user_model
from django.db import models
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

User = get_user_model()


class MicrosoftSSOUser(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    user_principal_name = models.CharField(max_length=255, null=True, blank=True)
    microsoft_id = models.CharField(max_length=255, blank=True, null=True)
    picture_raw = models.BinaryField(blank=True, null=True)
    locale = models.CharField(max_length=5, blank=True, null=True)

    @property
    def picture(self):
        if self.picture_raw:
            return mark_safe(
                '<img src = "data: image/png; base64, {}"'
                ' width="75" height="75">'.format(
                    b64encode(self.picture_raw).decode("utf8")
                )
            )
        return None

    def __str__(self):
        return f"{self.user.email} ({self.microsoft_id})"

    class Meta:
        db_table = "microsoft_sso_user"
        verbose_name = _("Microsoft SSO User")



from django.core.checks import Tags, register
from django.core.checks.messages import Warning

TEMPLATE_TAG_NAMES = ["show_form", "sso_tags"]


@register(Tags.templates)
def register_sso_check(app_configs, **kwargs):
    """Check for E003/W003 template warnings.

    This is a copy of the original check_for_template_tags_with_the_same_name
    but filtering out the TEMPLATE_TAG_NAMES from this library.

    Django will raise this warning if you're installed more than one SSO provider,
    like django_microsoft_sso and django_microsoft_sso.

    To silence any E003/W003 warning, you can add the following to your settings.py:
    SILENCED_SYSTEM_CHECKS = ["templates.W003"]  # or templates.E003 for Django<=5.1

    And to run an alternate version of this check,
    you can add the following to your settings.py:
    SSO_USE_ALTERNATE_W003 = True

    You need to silence the original templates.W003 check for this to work.
    New warnings will use the id `sso.W003`

    """
    try:  # Django <=5.0 error was templates.E003
        from django.core.checks.templates import (
            check_for_template_tags_with_the_same_name,
        )

        errors = check_for_template_tags_with_the_same_name(app_configs, **kwargs)
        errors = [
            Warning(msg=error.msg, hint=error.hint, obj=error.obj, id="sso.E003")
            for error in errors
            if not any(name in error.msg for name in TEMPLATE_TAG_NAMES)
        ]
        return errors
    except ImportError:  # Django >=5.1 error is now templates.W003
        from django.apps import apps
        from django.conf import settings
        from django.template.backends.django import DjangoTemplates

        errors = []
        if app_configs is None:
            app_configs = apps.get_app_configs()

        for config in app_configs:
            for engine in settings.TEMPLATES:
                if engine["BACKEND"] == "django.template.backends.django.DjangoTemplates":
                    engine_params = engine.copy()
                    engine_params.pop("BACKEND")
                    django_engine = DjangoTemplates(engine_params)
                    template_tag_errors = (
                        django_engine._check_for_template_tags_with_the_same_name()
                    )
                    for error in template_tag_errors:
                        if not any(name in error.msg for name in TEMPLATE_TAG_NAMES):
                            errors.append(
                                Warning(
                                    msg=error.msg,
                                    hint=error.hint,
                                    obj=error.obj,
                                    id="sso.W003",
                                )
                            )
        return errors



from django.urls import path

from django_microsoft_sso import conf, views

app_name = "django_microsoft_sso"

urlpatterns = []

if conf.MICROSOFT_SSO_ENABLED:
    urlpatterns += [
        path("login/", views.start_login, name="oauth_start_login"),
        path("callback/", views.callback, name="oauth_callback"),
    ]



import importlib
from copy import deepcopy

import pytest
from django.contrib.auth.models import User

from django_microsoft_sso import conf
from django_microsoft_sso.main import UserHelper

pytestmark = pytest.mark.django_db


def test_user_email(microsoft_response, callback_request):
    # Act
    helper = UserHelper(microsoft_response, callback_request)

    # Assert
    assert helper.user_email == "kalel@dailyplanet.com"


@pytest.mark.parametrize(
    "allowable_domains, expected_result", [(["dailyplanet.com"], True), ([], False)]
)
def test_email_is_valid(
    microsoft_response, callback_request, allowable_domains, expected_result, settings
):
    # Arrange
    settings.MICROSOFT_SSO_ALLOWABLE_DOMAINS = allowable_domains
    importlib.reload(conf)

    # Act
    helper = UserHelper(microsoft_response, callback_request)

    # Assert
    assert helper.email_is_valid == expected_result


@pytest.mark.parametrize("auto_create_super_user", [True, False])
def test_get_or_create_user(
    auto_create_super_user, microsoft_response, callback_request, settings
):
    # Arrange
    settings.MICROSOFT_SSO_AUTO_CREATE_FIRST_SUPERUSER = auto_create_super_user
    importlib.reload(conf)

    # Act
    helper = UserHelper(microsoft_response, callback_request)
    user = helper.get_or_create_user()

    # Assert
    assert user.first_name == microsoft_response["givenName"]
    assert user.last_name == microsoft_response["surname"]
    assert user.username == microsoft_response["userPrincipalName"]
    assert user.email == microsoft_response["mail"]
    assert user.is_active is True
    assert user.is_staff == auto_create_super_user
    assert user.is_superuser == auto_create_super_user


@pytest.mark.parametrize(
    "always_update_user_data, expected_is_equal", [(True, True), (False, False)]
)
def test_update_existing_user_record(
    always_update_user_data,
    callback_request,
    expected_is_equal,
    microsoft_response,
    microsoft_response_update,
    settings,
):
    # Arrange
    settings.MICROSOFT_SSO_ALWAYS_UPDATE_USER_DATA = always_update_user_data
    importlib.reload(conf)
    helper = UserHelper(microsoft_response, callback_request)
    helper.get_or_create_user()

    # Act
    helper = UserHelper(microsoft_response_update, callback_request)
    user = helper.get_or_create_user()

    # Assert
    assert (user.first_name == microsoft_response_update["givenName"]) == expected_is_equal
    assert (user.last_name == microsoft_response_update["surname"]) == expected_is_equal
    assert user.email == microsoft_response_update["mail"]


@pytest.mark.parametrize("use_email", [True, False])
def test_create_staff_from_list(
    microsoft_response, callback_request, settings, use_email, monkeypatch
):
    # Arrange
    monkeypatch.setattr(conf, "MICROSOFT_SSO_UNIQUE_EMAIL", use_email)
    settings.MICROSOFT_SSO_STAFF_LIST = [microsoft_response["mail"]]
    ms_response = deepcopy(microsoft_response)
    if not use_email:
        del ms_response["mail"]
        settings.MICROSOFT_SSO_STAFF_LIST = [microsoft_response["userPrincipalName"]]
    settings.MICROSOFT_SSO_AUTO_CREATE_FIRST_SUPERUSER = False
    importlib.reload(conf)

    # Act
    helper = UserHelper(ms_response, callback_request)
    helper.get_or_create_user()
    user = helper.find_user()

    # Assert
    assert user.is_active is True
    assert user.is_staff is True
    assert user.is_superuser is False


def test_add_all_users_to_staff_list(faker, microsoft_response, callback_request, settings):
    # Arrange
    settings.MICROSOFT_SSO_STAFF_LIST = ["*"]
    settings.MICROSOFT_SSO_AUTO_CREATE_FIRST_SUPERUSER = False
    importlib.reload(conf)

    emails = [
        faker.email(),
        faker.email(),
        faker.email(),
    ]

    # Act
    for email in emails:
        response = deepcopy(microsoft_response)
        response["mail"] = email
        response["userPrincipalName"] = email
        helper = UserHelper(response, callback_request)
        helper.get_or_create_user()
        helper.find_user()

    # Assert
    assert User.objects.filter(is_staff=True).count() == 3


def test_create_super_user_from_list(microsoft_response, callback_request, settings):
    # Arrange
    settings.MICROSOFT_SSO_AUTO_CREATE_FIRST_SUPERUSER = False
    settings.MICROSOFT_SSO_SUPERUSER_LIST = [microsoft_response["mail"]]
    importlib.reload(conf)

    # Act
    helper = UserHelper(microsoft_response, callback_request)
    user = helper.get_or_create_user()

    # Assert
    assert user.is_active is True
    assert user.is_staff is True
    assert user.is_superuser is True


@pytest.mark.parametrize(
    "unique_email",
    [True, False],
)
def test_create_user_without_email_address(
    microsoft_response, callback_request, unique_email, monkeypatch
):
    # Arrange
    microsoft_response_no_email = deepcopy(microsoft_response)
    del microsoft_response_no_email["mail"]
    helper = UserHelper(microsoft_response_no_email, callback_request)
    monkeypatch.setattr(conf, "MICROSOFT_SSO_UNIQUE_EMAIL", unique_email)

    if unique_email:
        # Act/Assert
        with pytest.raises(ValueError):
            helper.get_or_create_user()
    else:
        # Act
        helper.get_or_create_user()
        user = helper.find_user()

        # Assert
        assert user.email == ""
        assert user.username == microsoft_response_no_email["userPrincipalName"]


def test_different_null_values(microsoft_response, callback_request, monkeypatch):
    # Arrange
    microsoft_response_no_key = deepcopy(microsoft_response)
    del microsoft_response_no_key["mail"]
    microsoft_response_key_none = deepcopy(microsoft_response)
    microsoft_response_key_none["mail"] = None

    # Act
    no_key_helper = UserHelper(microsoft_response_no_key, callback_request)
    no_key_helper.get_or_create_user()
    user_one = no_key_helper.find_user()

    none_key_helper = UserHelper(microsoft_response_key_none, callback_request)
    none_key_helper.get_or_create_user()
    user_two = none_key_helper.find_user()

    # Assert
    assert user_one.email == ""
    assert user_one.username == microsoft_response_no_key["userPrincipalName"]
    assert user_two.email == ""
    assert user_two.username == microsoft_response_key_none["userPrincipalName"]


def test_duplicated_emails(microsoft_response, callback_request, settings):
    # Arrange
    User.objects.create(
        email=microsoft_response["mail"].upper(),
        username=microsoft_response["userPrincipalName"].upper(),
        first_name=microsoft_response["givenName"],
        last_name=microsoft_response["surname"],
    )

    lowercase_email_response = deepcopy(microsoft_response)
    lowercase_email_response["mail"] = lowercase_email_response["mail"].lower()
    lowercase_email_response["userPrincipalName"] = lowercase_email_response[
        "userPrincipalName"
    ].lower()
    uppercase_email_response = deepcopy(microsoft_response)
    uppercase_email_response["mail"] = uppercase_email_response["mail"].upper()
    uppercase_email_response["userPrincipalName"] = uppercase_email_response[
        "userPrincipalName"
    ].upper()

    # Act
    user_one_helper = UserHelper(uppercase_email_response, callback_request)
    user_one_helper.get_or_create_user()
    user_one = user_one_helper.find_user()

    user_two_helper = UserHelper(lowercase_email_response, callback_request)
    user_two_helper.get_or_create_user()
    user_two = user_two_helper.find_user()

    # Assert
    assert user_one.id == user_two.id
    assert user_one.email == user_two.email
    assert User.objects.count() == 1



from django.conf import settings
from loguru import logger

MICROSOFT_SSO_APPLICATION_ID = getattr(settings, "MICROSOFT_SSO_APPLICATION_ID", None)
MICROSOFT_SSO_CLIENT_SECRET = getattr(settings, "MICROSOFT_SSO_CLIENT_SECRET", None)
MICROSOFT_SSO_SCOPES = getattr(
    settings,
    "MICROSOFT_SSO_SCOPES",
    ["User.ReadBasic.All"],
)
MICROSOFT_SSO_TIMEOUT = getattr(settings, "MICROSOFT_SSO_TIMEOUT", 10)

MICROSOFT_SSO_ALLOWABLE_DOMAINS = getattr(settings, "MICROSOFT_SSO_ALLOWABLE_DOMAINS", [])
MICROSOFT_SSO_AUTO_CREATE_FIRST_SUPERUSER = getattr(
    settings, "MICROSOFT_SSO_AUTO_CREATE_FIRST_SUPERUSER", False
)
MICROSOFT_SSO_SESSION_COOKIE_AGE = getattr(
    settings, "MICROSOFT_SSO_SESSION_COOKIE_AGE", 3600
)
MICROSOFT_SSO_ENABLED = getattr(settings, "MICROSOFT_SSO_ENABLED", True)
MICROSOFT_SSO_SUPERUSER_LIST = getattr(settings, "MICROSOFT_SSO_SUPERUSER_LIST", [])
MICROSOFT_SSO_STAFF_LIST = getattr(settings, "MICROSOFT_SSO_STAFF_LIST", [])
MICROSOFT_SSO_CALLBACK_DOMAIN = getattr(settings, "MICROSOFT_SSO_CALLBACK_DOMAIN", None)
MICROSOFT_SSO_AUTO_CREATE_USERS = getattr(settings, "MICROSOFT_SSO_AUTO_CREATE_USERS", True)

MICROSOFT_SSO_AUTHENTICATION_BACKEND = getattr(
    settings, "MICROSOFT_SSO_AUTHENTICATION_BACKEND", None
)

MICROSOFT_SSO_PRE_VALIDATE_CALLBACK = getattr(
    settings,
    "MICROSOFT_SSO_PRE_VALIDATE_CALLBACK",
    "django_microsoft_sso.hooks.pre_validate_user",
)

MICROSOFT_SSO_PRE_CREATE_CALLBACK = getattr(
    settings,
    "MICROSOFT_SSO_PRE_CREATE_CALLBACK",
    "django_microsoft_sso.hooks.pre_create_user",
)

MICROSOFT_SSO_PRE_LOGIN_CALLBACK = getattr(
    settings,
    "MICROSOFT_SSO_PRE_LOGIN_CALLBACK",
    "django_microsoft_sso.hooks.pre_login_user",
)

MICROSOFT_SSO_LOGO_URL = getattr(
    settings,
    "MICROSOFT_SSO_LOGO_URL",
    "https://purepng.com/public/uploads/large/purepng.com-"
    "microsoft-logo-iconlogobrand-logoiconslogos-251519939091wmudn.png",
)

MICROSOFT_SSO_TEXT = getattr(settings, "MICROSOFT_SSO_TEXT", "Sign in with Microsoft")
MICROSOFT_SSO_NEXT_URL = getattr(settings, "MICROSOFT_SSO_NEXT_URL", "admin:index")
MICROSOFT_SSO_LOGIN_FAILED_URL = getattr(
    settings, "MICROSOFT_SSO_LOGIN_FAILED_URL", "admin:index"
)
MICROSOFT_SSO_SAVE_ACCESS_TOKEN = getattr(
    settings, "MICROSOFT_SSO_SAVE_ACCESS_TOKEN", False
)
MICROSOFT_SSO_ALWAYS_UPDATE_USER_DATA = getattr(
    settings, "MICROSOFT_SSO_ALWAYS_UPDATE_USER_DATA", False
)
MICROSOFT_SSO_LOGOUT_REDIRECT_PATH = getattr(
    settings, "MICROSOFT_SSO_LOGOUT_REDIRECT_PATH", "admin:index"
)
MICROSOFT_SLO_ENABLED = getattr(settings, "MICROSOFT_SLO_ENABLED", False)
SSO_USE_ALTERNATE_W003 = getattr(settings, "SSO_USE_ALTERNATE_W003", False)

if SSO_USE_ALTERNATE_W003:
    from django_microsoft_sso.checks.warnings import register_sso_check  # noqa

MICROSOFT_SSO_AUTHORITY = getattr(settings, "MICROSOFT_SSO_AUTHORITY", None)
MICROSOFT_SSO_UNIQUE_EMAIL = getattr(settings, "MICROSOFT_SSO_UNIQUE_EMAIL", False)
MICROSOFT_SSO_ENABLE_LOGS = getattr(settings, "MICROSOFT_SSO_ENABLE_LOGS", True)
MICROSOFT_SSO_ENABLE_MESSAGES = getattr(settings, "MICROSOFT_SSO_ENABLE_MESSAGES", True)
MICROSOFT_SSO_SAVE_BASIC_MICROSOFT_INFO = getattr(
    settings, "MICROSOFT_SSO_SAVE_BASIC_MICROSOFT_INFO", True
)
MICROSOFT_SSO_SHOW_FAILED_LOGIN_MESSAGE = getattr(
    settings, "MICROSOFT_SSO_SHOW_FAILED_LOGIN_MESSAGE", False
)
MICROSOFT_SSO_GRAPH_TIMEOUT = getattr(settings, "MICROSOFT_SSO_GRAPH_TIMEOUT", 10)

if MICROSOFT_SSO_ENABLE_LOGS:
    logger.enable("django_microsoft_sso")
else:
    logger.disable("django_microsoft_sso")



# Generated by Django 4.2 on 2023-04-15 03:34

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='MicrosoftSSOUser',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('microsoft_id', models.CharField(blank=True, max_length=255, null=True)),
                ('picture_raw', models.BinaryField(blank=True, null=True)),
                ('locale', models.CharField(blank=True, max_length=5, null=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Microsoft SSO User',
                'db_table': 'microsoft_sso_user',
            },
        ),
    ]



import importlib
import re

from django import template
from django.conf import settings
from django.templatetags.static import static
from django.urls import reverse
from django.utils.translation import gettext
from loguru import logger

register = template.Library()


@register.simple_tag
def define_sso_providers():
    provider_pattern = re.compile(r"^django_(.+)_sso$")
    providers = []
    for app in settings.INSTALLED_APPS:
        match = re.search(provider_pattern, app)
        if match:
            providers.append(match.group(1))

    sso_providers = []
    for provider in providers:
        package_name = f"django_{provider}_sso"
        try:
            package = importlib.import_module(package_name)
            conf = getattr(package, "conf")
            if getattr(conf, f"{provider.upper()}_SSO_ENABLED"):
                sso_providers.append(
                    {
                        "name": provider,
                        "logo_url": getattr(conf, f"{provider.upper()}_SSO_LOGO_URL"),
                        "text": gettext(getattr(conf, f"{provider.upper()}_SSO_TEXT")),
                        "login_url": reverse(f"django_{provider}_sso:oauth_start_login"),
                        "css_url": static(f"django_{provider}_sso/{provider}_button.css"),
                    }
                )
        except Exception as e:
            logger.error(f"Error importing {package_name}: {e}")
    return sso_providers



# Generated by Django 5.0.4 on 2024-04-09 16:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_microsoft_sso', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='microsoftssouser',
            name='user_principal_name',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
